@@ -1,6 +1,6 @@
-from .backends.interface import LockedException  # noqa
+from ._cache_condition import NOT_NONE  # noqa
+from .backends.interface import CacheBackendInteractionException, LockedException  # noqa
 from .decorators import (  # noqa
-    NOT_NONE,
     CacheDetect,
     CircuitBreakerOpen,
     RateLimitException,
@@ -8,7 +8,7 @@ from .decorators import (  # noqa
     fast_condition,
 )
 from .formatter import default_formatter, get_template_and_func_for, get_template_for_key  # noqa
-from .helpers import add_prefix, all_keys_lower  # noqa
+from .helpers import add_prefix, all_keys_lower, memory_limit  # noqa
 from .key import noself  # noqa
 from .validation import set_invalidate_further  # noqa
 from .wrapper import Cache  # noqa
@@ -21,6 +21,7 @@ rate_limit = cache.rate_limit
 failover = cache.failover
 circuit_breaker = cache.circuit_breaker
 early = cache.early
+soft = cache.soft
 hit = cache.hit
 dynamic = cache.dynamic
 locked = cache.locked
