@@ -1,9 +1,11 @@
+from .._cache_condition import NOT_NONE
 from .bloom import bloom, dual_bloom  # noqa
-from .cache.defaults import NOT_NONE, CacheDetect, context_cache_detect  # noqa
+from .cache.defaults import CacheDetect, context_cache_detect  # noqa
 from .cache.early import early  # noqa
 from .cache.fail import failover, fast_condition  # noqa
 from .cache.hit import hit  # noqa
 from .cache.simple import cache  # noqa
+from .cache.soft import soft  # noqa
 from .circuit_breaker import CircuitBreakerOpen, circuit_breaker  # noqa
 from .locked import locked  # noqa
 from .rate import RateLimitException, rate_limit  # noqa
