@@ -8,7 +8,7 @@ from .formatter import get_templates_for_func, template_to_pattern
 from .key import get_call_values, get_func_params
 
 
-async def invalidate_func(backend: Backend, func, kwargs: Optional[Dict] = None):
+async def invalidate_func(backend: Backend, func, kwargs: Optional[Dict] = None) -> None:
     values = {**{param: "*" for param in get_func_params(func)}, **kwargs}
     for template in get_templates_for_func(func):
         del_template = template_to_pattern(template, **values)
@@ -36,9 +36,8 @@ def invalidate(
             if callable(target):
                 asyncio.create_task(invalidate_func(backend, target, _args))
             else:
-                asyncio.create_task(
-                    backend.delete_match(target.format({k: str(v) if v is not None else "" for k, v in _args.items()}))
-                )
+                key = target.format(**{k: str(v) if v is not None else "" for k, v in _args.items()})
+                asyncio.create_task(backend.delete_match(key))
             return result
 
         return _wrap
@@ -57,7 +56,7 @@ async def _invalidate_middleware(call, *args, key=None, backend=None, cmd=None,
     if _INVALIDATE_FURTHER.get() and key is not None and cmd != "delete":
         asyncio.create_task(backend.delete(key))
         _INVALIDATE_FURTHER.set(False)
-        return None
+        return kwargs.get("default")
     if key is None:
         return await call(*args, **kwargs)
     return await call(*args, key=key, **kwargs)
