@@ -1,10 +1,11 @@
 import asyncio
 from contextlib import contextmanager
 from functools import partial, wraps
-from typing import Any, Callable, Dict, Iterable, Optional, Tuple, Type, Union
-from urllib.parse import parse_qsl, urlparse
+from typing import Any, AsyncIterator, Callable, Dict, Iterable, Mapping, Optional, Tuple, Type, Union
 
 from . import decorators, validation
+from ._cache_condition import create_time_condition, get_cache_condition
+from ._settings import settings_url_parse
 from ._typing import TTL, CacheCondition
 from .backends.interface import Backend
 from .backends.memory import Memory
@@ -12,31 +13,19 @@ from .disable_control import ControlMixin, _is_disable_middleware
 from .key import ttl_to_seconds
 
 try:
-    import aioredis
-except ImportError:
-    BcastClientSide, IndexRedis, Redis = None, None, None
-else:
     from .backends.client_side import BcastClientSide
-    from .backends.index import IndexRedis
-    from .backends.redis import Redis
-
-    del aioredis
+except ImportError:
+    BcastClientSide = None
 
 try:
-    import diskcache
+    from .backends.diskcache import DiskCache
 except ImportError:
     DiskCache = None
-else:
-    from .backends.diskcache import DiskCache
 
 
 #  pylint: disable=too-many-public-methods
 
 
-class BackendNotAvailable(Exception):
-    pass
-
-
 def _create_auto_init():
     lock = asyncio.Lock()
 
@@ -55,8 +44,8 @@ def _create_auto_init():
 class Cache(Backend):
     default_prefix = ""
 
-    def __init__(self, name=None):
-        self._backends = {}  # {key: (backend, middleware)}
+    def __init__(self, name: Optional[str] = None):
+        self._backends: Dict[str, Tuple[Backend, Tuple[Callable]]] = {}  # {key: (backend, middleware)}
         self._default_middlewares = (
             _is_disable_middleware,
             _create_auto_init(),
@@ -71,56 +60,54 @@ class Cache(Backend):
     def set_default_fail_exceptions(self, *exc: Type[Exception]):
         self._default_fail_exceptions = exc
 
-    def disable(self, *cmds, prefix=""):
+    def disable(self, *cmds: str, prefix: str = ""):
         return self._get_backend(prefix).disable(*cmds)
 
-    def disable_all(self, *cmds):
+    def disable_all(self, *cmds: str):
         for backend, _ in self._backends.values():
             backend.disable(*cmds)
 
-    def enable(self, *cmds, prefix=""):
+    def enable(self, *cmds: str, prefix: str = ""):
         return self._get_backend(prefix).enable(*cmds)
 
-    def enable_all(self, *cmds):
+    def enable_all(self, *cmds: str):
         for backend, _ in self._backends.values():
             backend.enable(*cmds)
 
     @contextmanager
-    def disabling(self, *cmds, prefix=""):
+    def disabling(self, *cmds: str, prefix: str = ""):
         self.disable(*cmds, prefix=prefix)
         yield
         self.enable(*cmds, prefix=prefix)
 
-    def is_disable(self, *cmds, prefix=""):
+    def is_disable(self, *cmds: str, prefix: str = ""):
         return self._get_backend(prefix).is_disable(*cmds)
 
-    def is_enable(self, *cmds, prefix=""):
+    def is_enable(self, *cmds: str, prefix: str = ""):
         return not self.is_disable(*cmds, prefix=prefix)
 
-    def _get_backend_and_config(self, key) -> Tuple[Backend, Tuple[Callable]]:
+    def _get_backend_and_config(self, key: str) -> Tuple[Backend, Tuple[Callable]]:
         for prefix in sorted(self._backends.keys(), reverse=True):
             if key.startswith(prefix):
                 return self._backends[prefix]
         return self._backends[self.default_prefix]
 
-    def _get_backend(self, key) -> Backend:
+    def _get_backend(self, key: str) -> Backend:
         backend, _ = self._get_backend_and_config(key)
         return backend
 
-    def setup(self, settings_url: str, middlewares: Tuple = (), prefix=default_prefix, **kwargs):
+    def setup(self, settings_url: str, middlewares: Tuple = (), prefix: str = default_prefix, **kwargs):
         params = settings_url_parse(settings_url)
         params.update(kwargs)
 
         if params.pop("client_side", None):
             params["backend"] = BcastClientSide
-        if "index_name" in params:
-            params["backend"] = IndexRedis
         backend = params.pop("backend")
 
         self._add_backend(backend, middlewares, prefix, **params)
         return self._backends[prefix][0]
 
-    def _add_backend(self, backend_class, middlewares=(), prefix=default_prefix, **params):
+    def _add_backend(self, backend_class: Type[Backend], middlewares=(), prefix: str = default_prefix, **params):
         class _backend_class(ControlMixin, backend_class):
             pass
 
@@ -135,6 +122,13 @@ class Cache(Backend):
         for backend, _ in self._backends.values():
             await backend.init()
 
+    @property
+    def is_init(self) -> bool:
+        for backend, _ in self._backends.values():
+            if not backend.is_init:
+                return False
+        return True
+
     def _with_middlewares(self, cmd: str, key):
         backend, middlewares = self._get_backend_and_config(key)
         return self._with_middlewares_for_backend(cmd, backend, middlewares)
@@ -145,25 +139,27 @@ class Cache(Backend):
             call = partial(middleware, call, cmd=cmd, backend=backend)
         return call
 
-    def set(
+    async def set(
         self,
         key: str,
         value: Any,
-        expire: Union[float, None, TTL] = None,
+        expire: Union[float, TTL, None] = None,
         exist: Optional[bool] = None,
-    ):
-        return self._with_middlewares("set", key)(key=key, value=value, expire=ttl_to_seconds(expire), exist=exist)
+    ) -> bool:
+        return await self._with_middlewares("set", key)(
+            key=key, value=value, expire=ttl_to_seconds(expire), exist=exist
+        )
 
-    def set_raw(self, key: str, value: Any, **kwargs):
-        return self._with_middlewares("set_raw", key)(key=key, value=value, **kwargs)
+    async def set_raw(self, key: str, value: Any, **kwargs):
+        return await self._with_middlewares("set_raw", key)(key=key, value=value, **kwargs)
 
-    def get(self, key: str, default: Optional[Any] = None) -> Any:
-        return self._with_middlewares("get", key)(key=key, default=default)
+    async def get(self, key: str, default: Optional[Any] = None) -> Any:
+        return await self._with_middlewares("get", key)(key=key, default=default)
 
-    def get_raw(self, key: str) -> Any:
-        return self._with_middlewares("get_raw", key)(key=key)
+    async def get_raw(self, key: str) -> Any:
+        return await self._with_middlewares("get_raw", key)(key=key)
 
-    async def keys_match(self, pattern: str):
+    async def keys_match(self, pattern: str) -> AsyncIterator[str]:
         backend, middlewares = self._get_backend_and_config(pattern)
 
         async def call(_pattern):
@@ -174,45 +170,86 @@ class Cache(Backend):
         async for key in (await call(pattern)):
             yield key
 
-    def get_many(self, *keys: str):
-        return self._with_middlewares("get_many", keys[0])(*keys)
+    async def scan(self, pattern: str, batch_size: int = 100) -> AsyncIterator[str]:
+        backend, middlewares = self._get_backend_and_config(pattern)
 
-    def get_bits(self, key: str, *indexes: int, size: int = 1):
-        return self._with_middlewares("get_bits", key)(key, *indexes, size=size)
+        async def call(_pattern):
+            return backend.scan(_pattern)
 
-    def incr_bits(self, key: str, *indexes: int, size: int = 1, by: int = 1):
-        return self._with_middlewares("incr_bits", key)(key, *indexes, size=size, by=by)
+        for middleware in middlewares:
+            call = partial(middleware, call, cmd="scan", backend=backend)
+        async for key in (await call(pattern)):
+            yield key
 
-    def incr(self, key: str) -> int:
-        return self._with_middlewares("incr", key)(key=key)
+    async def get_match(
+        self, pattern: str, batch_size: int = 100, default: Optional[Any] = None
+    ) -> AsyncIterator[Tuple[str, Any]]:
+        backend, middlewares = self._get_backend_and_config(pattern)
 
-    def delete(self, key: str):
-        return self._with_middlewares("delete", key)(key=key)
+        async def call(_pattern, _batch_size, _default):
+            return backend.get_match(_pattern, batch_size=_batch_size, default=_default)
 
-    def delete_match(self, pattern: str):
-        return self._with_middlewares("delete_match", pattern)(pattern=pattern)
+        for middleware in middlewares:
+            call = partial(middleware, call, cmd="get_match", backend=backend)
+        async for key, value in (await call(pattern, batch_size, default)):
+            yield key, value
 
-    def expire(self, key: str, timeout: TTL):
-        return self._with_middlewares("expire", key)(key=key, timeout=ttl_to_seconds(timeout))
+    async def get_many(self, *keys: str, default: Optional[Any] = None) -> Tuple[Any]:
+        backends = {}
+        for key in keys:
+            backend = self._get_backend(key)
+            backends.setdefault(backend, []).append(key)
+        result = {}
+        for _keys in backends.values():
+            _values = await self._with_middlewares("get_many", _keys[0])(*_keys, default=default)
+            result.update(dict(zip(_keys, _values)))
+        return tuple(result.get(key) for key in keys)
 
-    def get_expire(self, key: str):
-        return self._with_middlewares("get_expire", key)(key=key)
+    async def set_many(self, pairs: Mapping[str, Any], expire: Union[float, TTL, None] = None):
+        backends = {}
+        for key in pairs:
+            backend = self._get_backend(key)
+            backends.setdefault(backend, []).append(key)
+        for backend, keys in backends.items():
+            data = {key: pairs[key] for key in keys}
+            await self._with_middlewares("set_many", keys[0])(data, expire=ttl_to_seconds(expire))
 
-    def exists(self, key: str):
-        return self._with_middlewares("exists", key)(key=key)
+    async def get_bits(self, key: str, *indexes: int, size: int = 1) -> Tuple[int]:
+        return await self._with_middlewares("get_bits", key)(key, *indexes, size=size)
 
-    def set_lock(self, key: str, value: Any, expire: TTL):
-        return self._with_middlewares("set_lock", key)(key=key, value=value, expire=ttl_to_seconds(expire))
+    async def incr_bits(self, key: str, *indexes: int, size: int = 1, by: int = 1):
+        return await self._with_middlewares("incr_bits", key)(key, *indexes, size=size, by=by)
 
-    def unlock(self, key: str, value: str):
-        return self._with_middlewares("unlock", key)(key=key, value=value)
+    async def incr(self, key: str) -> int:
+        return await self._with_middlewares("incr", key)(key=key)
 
-    def listen(self, pattern: str, *cmds, reader=None):
-        return self._with_middlewares("listen", pattern)(pattern, *cmds, reader=reader)
+    async def delete(self, key: str):
+        return await self._with_middlewares("delete", key)(key=key)
 
-    def ping(self, message: Optional[bytes] = None) -> str:
+    async def delete_match(self, pattern: str):
+        return await self._with_middlewares("delete_match", pattern)(pattern=pattern)
+
+    async def expire(self, key: str, timeout: TTL):
+        return await self._with_middlewares("expire", key)(key=key, timeout=ttl_to_seconds(timeout))
+
+    async def get_expire(self, key: str):
+        return await self._with_middlewares("get_expire", key)(key=key)
+
+    async def exists(self, key: str):
+        return await self._with_middlewares("exists", key)(key=key)
+
+    async def set_lock(self, key: str, value: Any, expire: TTL):
+        return await self._with_middlewares("set_lock", key)(key=key, value=value, expire=ttl_to_seconds(expire))
+
+    async def unlock(self, key: str, value: str):
+        return await self._with_middlewares("unlock", key)(key=key, value=value)
+
+    async def get_size(self, key: str):
+        return await self._with_middlewares("get_size", key)(key)
+
+    async def ping(self, message: Optional[bytes] = None) -> str:
         message = b"PING" if message is None else message
-        return self._with_middlewares("ping", message.decode())(message=message)
+        return await self._with_middlewares("ping", message.decode())(message=message)
 
     async def clear(self):
         for backend, _ in self._backends.values():
@@ -222,21 +259,26 @@ class Cache(Backend):
         for backend, _ in self._backends.values():
             backend.close()
 
-    def is_locked(
+    async def is_locked(
         self,
         key: str,
         wait: Union[float, None, TTL] = None,
         step: Union[int, float] = 0.1,
     ) -> bool:
-        return self._with_middlewares("is_locked", key)(key=key, wait=ttl_to_seconds(wait), step=step)
+        return await self._with_middlewares("is_locked", key)(key=key, wait=ttl_to_seconds(wait), step=step)
 
     def _wrap_on(self, decorator_fabric, upper, **decor_kwargs):
         if upper:
             return self._wrap_with_condition(decorator_fabric, **decor_kwargs)
         return self._wrap(decorator_fabric, **decor_kwargs)
 
-    def _wrap(self, decorator_fabric, lock=False, **decor_kwargs):
+    def _wrap(self, decorator_fabric, lock=False, time_condition=None, **decor_kwargs):
         def _decorator(func):
+            if time_condition is not None:
+                condition, _decor = create_time_condition(time_condition)
+                func = _decor(func)
+                decor_kwargs["condition"] = condition
+
             decorator = decorator_fabric(self, **decor_kwargs)(func)
 
             @wraps(func)
@@ -252,8 +294,12 @@ class Cache(Backend):
 
         return _decorator
 
-    def _wrap_with_condition(self, decorator_fabric, condition, lock=False, **decor_kwargs):
+    def _wrap_with_condition(self, decorator_fabric, condition, lock=False, time_condition=None, **decor_kwargs):
         def _decorator(func):
+            _condition = condition
+            if time_condition is not None:
+                _condition, _decor = create_time_condition(time_condition)
+                func = _decor(func)
             decorator_fabric(self, **decor_kwargs)(func)  # to register cache templates
 
             @wraps(func)
@@ -263,7 +309,7 @@ class Cache(Backend):
                     def new_condition(result, _args, _kwargs, key):
                         if detect.keys:
                             return False
-                        return condition(result, _args, _kwargs, key=key) if condition else result is not None
+                        return _condition(result, _args, _kwargs, key=key) if _condition else result is not None
 
                     decorator = decorator_fabric(self, **decor_kwargs, condition=new_condition)
                     if lock:
@@ -285,6 +331,7 @@ class Cache(Backend):
         ttl: TTL,
         key: Optional[str] = None,
         condition: CacheCondition = None,
+        time_condition: Optional[TTL] = None,
         prefix: str = "",
         upper: bool = False,
         lock: bool = False,
@@ -295,7 +342,8 @@ class Cache(Backend):
             lock=lock,
             ttl=ttl_to_seconds(ttl),
             key=key,
-            condition=condition,
+            condition=get_cache_condition(condition),
+            time_condition=ttl_to_seconds(time_condition),
             prefix=prefix,
         )
 
@@ -307,6 +355,7 @@ class Cache(Backend):
         exceptions: Union[Type[Exception], Iterable[Type[Exception]], None] = None,
         key: Optional[str] = None,
         condition: CacheCondition = None,
+        time_condition: Optional[TTL] = None,
         prefix: str = "fail",
     ):
         exceptions = exceptions or self._default_fail_exceptions
@@ -315,7 +364,8 @@ class Cache(Backend):
             ttl=ttl_to_seconds(ttl),
             exceptions=exceptions,
             key=key,
-            condition=condition,
+            condition=get_cache_condition(condition),
+            time_condition=ttl_to_seconds(time_condition),
             prefix=prefix,
         )
 
@@ -325,6 +375,7 @@ class Cache(Backend):
         key: Optional[str] = None,
         early_ttl: Optional[TTL] = None,
         condition: CacheCondition = None,
+        time_condition: Optional[TTL] = None,
         prefix: str = "early",
         upper: bool = False,
     ):
@@ -334,7 +385,31 @@ class Cache(Backend):
             ttl=ttl_to_seconds(ttl),
             key=key,
             early_ttl=ttl_to_seconds(early_ttl),
-            condition=condition,
+            condition=get_cache_condition(condition),
+            time_condition=ttl_to_seconds(time_condition),
+            prefix=prefix,
+        )
+
+    def soft(
+        self,
+        ttl: TTL,
+        key: Optional[str] = None,
+        soft_ttl: Optional[TTL] = None,
+        exceptions: Union[Type[Exception], Tuple[Type[Exception]]] = Exception,
+        condition: CacheCondition = None,
+        time_condition: Optional[TTL] = None,
+        prefix: str = "soft",
+        upper: bool = False,
+    ):
+        return self._wrap_on(
+            decorators.soft,
+            upper,
+            ttl=ttl_to_seconds(ttl),
+            key=key,
+            soft_ttl=ttl_to_seconds(soft_ttl),
+            exceptions=exceptions,
+            condition=get_cache_condition(condition),
+            time_condition=ttl_to_seconds(time_condition),
             prefix=prefix,
         )
 
@@ -345,6 +420,7 @@ class Cache(Backend):
         update_after: int = 0,
         key: Optional[str] = None,
         condition: CacheCondition = None,
+        time_condition: Optional[TTL] = None,
         prefix: str = "hit",
         upper: bool = False,
     ):
@@ -355,7 +431,8 @@ class Cache(Backend):
             cache_hits=cache_hits,
             update_after=update_after,
             key=key,
-            condition=condition,
+            condition=get_cache_condition(condition),
+            time_condition=ttl_to_seconds(time_condition),
             prefix=prefix,
         )
 
@@ -364,6 +441,7 @@ class Cache(Backend):
         ttl: TTL = 60 * 60 * 24,
         key: Optional[str] = None,
         condition: CacheCondition = None,
+        time_condition: Optional[TTL] = None,
         prefix: str = "dynamic",
         upper: bool = False,
     ):
@@ -374,7 +452,8 @@ class Cache(Backend):
             cache_hits=3,
             update_after=1,
             key=key,
-            condition=condition,
+            condition=get_cache_condition(condition),
+            time_condition=ttl_to_seconds(time_condition),
             prefix=prefix,
         )
 
@@ -486,48 +565,3 @@ class Cache(Backend):
             capacity=capacity,
             prefix=prefix,
         )
-
-
-def _fix_params_types(params: Dict[str, str]) -> Dict[str, Union[str, int, bool, float]]:
-    new_params = {}
-    bool_keys = ("safe", "enable", "disable", "client_side")
-    true_values = (
-        "1",
-        "true",
-    )
-    for key, value in params.items():
-        if key.lower() in bool_keys:
-            value = value.lower() in true_values
-        elif value.isdigit():
-            value = int(value)
-        else:
-            try:
-                value = float(value)
-            except ValueError:
-                pass
-        new_params[key.lower()] = value
-    return new_params
-
-
-def settings_url_parse(url):
-    params = {}
-    parse_result = urlparse(url)
-    params.update(dict(parse_qsl(parse_result.query)))
-    params = _fix_params_types(params)
-    if parse_result.scheme == "redis" or parse_result.scheme == "rediss":
-        if Redis is None:
-            raise BackendNotAvailable("Redis backend requires `aioredis` to be installed.")
-        params["backend"] = Redis
-        params["address"] = parse_result._replace(query=None)._replace(fragment=None).geturl()
-    elif parse_result.scheme == "mem":
-        params["backend"] = Memory
-    elif parse_result.scheme == "disk":
-        if DiskCache is None:
-            raise BackendNotAvailable("Disk backend requires `diskcache` to be installed.")
-        params["backend"] = DiskCache
-    elif parse_result.scheme == "":
-        params["backend"] = Memory
-        params["disable"] = True
-    else:
-        raise BackendNotAvailable(f"wrong backend alias {parse_result.scheme}")
-    return params
