@@ -1,7 +1,7 @@
 from __future__ import annotations
 
 import os
-from typing import TYPE_CHECKING, Type
+from typing import TYPE_CHECKING
 
 import pytest
 
@@ -29,10 +29,10 @@ def redis_dsn():
 
 
 @pytest.fixture
-async def backend_factory():
+def backend_factory():
     backend = None
 
-    async def factory(backend_cls: Type[Backend], *args, **kwargs):
+    async def factory(backend_cls: type[Backend], *args, **kwargs):
         nonlocal backend
 
         backend = backend_cls(*args, **kwargs)
