@@ -1,16 +1,16 @@
 import asyncio
 import sys
-from unittest.mock import Mock
 
 import pytest
+import pytest_asyncio
 
-from cashews.backends.interface import Backend, ProxyBackend
+from cashews.backends.interface import Backend
 from cashews.backends.memory import Memory
 
 pytestmark = pytest.mark.asyncio
 
 
-@pytest.fixture(
+@pytest_asyncio.fixture(
     name="cache",
     params=[
         "memory",
@@ -51,6 +51,12 @@ async def test_set_exist(cache):
     assert not await cache.set("key2", b"value", exist=False)
 
 
+async def test_set_many(cache):
+    await cache.set_many({"key1": "value1", "key2": "value2"})
+    assert await cache.get("key1", "value1")
+    assert await cache.get("key2", "value2")
+
+
 async def test_get_no_value(cache):
     assert await cache.get("key2") is None
 
@@ -93,35 +99,6 @@ async def test_get_set_expire(cache):
     assert await cache.get_expire("key") == 1
 
 
-@pytest.mark.parametrize(
-    ("method", "args", "defaults"),
-    (
-        ("get", ("key",), {"default": None}),
-        ("get_raw", ("key",), None),
-        ("set", ("key", "value"), {"exist": None, "expire": None}),
-        ("set_raw", ("key", "value"), None),
-        ("incr", ("key",), None),
-        ("exists", ("key",), None),
-        ("delete", ("key",), None),
-        ("expire", ("key", 10), None),
-        ("ping", (), None),
-        ("clear", (), None),
-        ("set_lock", ("key", "value", 10), None),
-        ("unlock", ("key", "value"), None),
-        ("is_locked", ("key",), {"wait": None, "step": 0.1}),
-    ),
-)
-async def test_proxy_backend(method, args, defaults):
-    target = Mock(wraps=Backend())
-    backend = ProxyBackend(target=target)
-
-    await getattr(backend, method)(*args)
-    if defaults:
-        getattr(target, method).assert_called_once_with(*args, **defaults)
-    else:
-        getattr(target, method).assert_called_once_with(*args)
-
-
 async def test_delete_match(cache: Backend):
     await cache.set("pref:test:test", b"value")
     await cache.set("pref:value:test", b"value2")
@@ -142,7 +119,7 @@ async def test_delete_match(cache: Backend):
     assert await cache.get("pref:test:tests") is not None
 
 
-async def test_keys_match(cache: Backend):
+async def test_scan(cache: Backend):
     await cache.set("pref:test:test", b"value")
     await cache.set("pref:value:test", b"value2")
     await cache.set("pref:-:test", b"-")
@@ -151,18 +128,42 @@ async def test_keys_match(cache: Backend):
     await cache.set("ppref:test:test", b"value3")
     await cache.set("pref:test:tests", b"value3")
 
-    keys = [key async for key in cache.keys_match("pref:*:test")]
+    keys = [key async for key in cache.scan("pref:*:test")]
 
     assert len(keys) == 4
+    assert set(keys) == {"pref:test:test", "pref:value:test", "pref:-:test", "pref:*:test"}
+
+
+async def test_get_match(cache: Backend):
+    await cache.set("pref:test:test", b"value")
+    await cache.set("pref:value:test", b"value2")
+    await cache.set("pref:-:test", b"-")
+    await cache.set("pref:*:test", b"*")
+
+    await cache.set("ppref:test:test", b"value3")
+    await cache.set("pref:test:tests", b"value3")
+
+    match = [(key, value) async for key, value in cache.get_match("pref:*:test")]
+
+    assert len(match) == 4
+    assert dict(match) == {
+        "pref:test:test": b"value",
+        "pref:value:test": b"value2",
+        "pref:-:test": b"-",
+        "pref:*:test": b"*",
+    }
+    match = [(key, value) async for key, value in cache.get_match("not_exists:*")]
+    assert len(match) == 0
 
 
 async def test_get_size(cache: Backend):
     await cache.set("test", b"1")
     assert await cache.get_size("test") in (
-        sys.getsizeof((None, b"1")) + sys.getsizeof(b"1") + sys.getsizeof(None),
-        68,
-        -1,
-    )  # 106 - ordered dict,  68 redis, -1 for diskcache
+        sys.getsizeof((None, b"1")) + sys.getsizeof(b"1") + sys.getsizeof(None),  # ordered dict
+        66,  # redis 6
+        72,  # redis 7
+        -1,  # diskcache
+    )
 
 
 async def test_get_bits(cache: Backend):
@@ -204,7 +205,7 @@ async def test_lru(backend_factory):
         assert await cache.get(f"key:{i}") == i
 
     for i in range(6, 10):
-        assert await cache.get(f"key:{i}") == None
+        assert await cache.get(f"key:{i}") is None
 
 
 async def test_lru2(backend_factory):
@@ -224,7 +225,7 @@ async def test_lru2(backend_factory):
     assert len(cache.store) == 10
 
     for i in range(5):
-        assert await cache.get(f"key:{i}") == None
+        assert await cache.get(f"key:{i}") is None
 
     for i in range(6, 10):
         assert await cache.get(f"key:{i}") == i
