@@ -1,6 +1,7 @@
 from datetime import timedelta
 
 import pytest
+import pytest_asyncio
 
 from cashews import mem
 
@@ -34,7 +35,7 @@ def _app():
     return app
 
 
-@pytest.fixture(name="cli")
+@pytest_asyncio.fixture(name="cli")
 async def _cli(aiohttp_client, app):
     return await aiohttp_client(app)
 
