@@ -8,13 +8,12 @@ from cashews.validation import invalidate_func, set_invalidate_further
 
 pytestmark = pytest.mark.asyncio
 
-_cache_ = Cache()
-
 
 @pytest.fixture(name="cache")
-async def _cache():
-    _cache_.setup("mem://")
-    return _cache_
+def _cache():
+    cache = Cache()
+    cache.setup("mem://")
+    return cache
 
 
 async def test_invalidate_func(cache: Cache):
@@ -61,6 +60,24 @@ async def test_invalidate_decor(cache: Cache):
     assert first_call != await func("test")
 
 
+async def test_invalidate_decor_str(cache: Cache):
+    @cache(ttl=1, key="key:{arg}")
+    async def func(arg):
+        return random.random()
+
+    @cache.invalidate("key:{arg}")
+    async def func2(arg, default=True):
+        return random.random()
+
+    first_call = await func("test")
+    await asyncio.sleep(0)
+
+    assert first_call == await func("test")
+    await func2("test")
+    await asyncio.sleep(0)
+    assert first_call != await func("test")
+
+
 async def test_invalidate_decor_complicate(cache: Cache):
     @cache(ttl=1, key="arg-{arg}:{key}:{flag}")
     async def func(arg, key=b"test", flag=True, **kwargs):
@@ -117,7 +134,9 @@ async def test_invalidate_further(cache):
     first = await func()
     assert await func() == first
 
-    assert await asyncio.create_task(clear()) != first  # the key was deleted
+    on_clear = await asyncio.create_task(clear())
+    assert on_clear is not None
+    assert on_clear != first  # the key was deleted
     second = await func()
     assert second != first
     assert await func() == second
