@@ -4,6 +4,9 @@ from datetime import date, datetime, timedelta
 from decimal import Decimal
 
 import pytest
+import pytest_asyncio
+from hypothesis import example, given, settings
+from hypothesis import strategies as st
 
 from cashews.serialize import PickleSerializerMixin, UnSecureDataError
 
@@ -33,15 +36,26 @@ class DummyCache:
         for key in keys:
             del self.store[key]
 
-    async def get_many(self, *keys):
-        return [self.store.get(key, None) for key in keys]
+    async def get_many(self, *keys, default=None):
+        return [self.store.get(key, default) for key in keys]
+
+    async def set_many(self, pairs, **kwargs):
+        self.store.update(pairs)
 
 
 class Cache(PickleSerializerMixin, DummyCache):
     pass
 
 
-@pytest.fixture(name="cache", params=["dummy", pytest.param("redis", marks=pytest.mark.redis)])
+@pytest_asyncio.fixture(
+    name="cache",
+    params=[
+        "default",
+        pytest.param("redis", marks=pytest.mark.redis),
+        pytest.param("dill", marks=pytest.mark.integration),
+        pytest.param("sqlalchemy", marks=pytest.mark.integration),
+    ],
+)
 async def _cache(request, redis_dsn):
     if request.param == "redis":
         from cashews.backends.redis import Redis
@@ -50,7 +64,7 @@ async def _cache(request, redis_dsn):
         await redis.init()
         await redis.clear()
         return redis
-    return Cache(hash_key="test", digestmod="md5")
+    return Cache(hash_key="test", digestmod="md5", pickle_type=request.param)
 
 
 @pytest.mark.parametrize(
@@ -76,8 +90,10 @@ async def _cache(request, redis_dsn):
 )
 async def test_serialize_simple_value(value, cache):
     await cache.set("key", value)
+    await cache.set_many({"key1": value, "key2": value})
     assert await cache.get("key") == value
-    assert await cache.get_many("key") == (value,)
+    assert await cache.get("key2") == value
+    assert await cache.get_many("key1", "key2") == (value, value)
 
 
 @pytest.mark.parametrize("value", (NT(name="test", test="lol"), NT()))
@@ -147,14 +163,12 @@ async def test_unsecure_value(value, cache):
     await cache.set_raw("key", value)
     with pytest.raises(UnSecureDataError):
         await cache.get("key")
-    assert not await cache.get_raw("key")
 
 
 async def test_unsecure_value_many(cache):
     await cache.set_raw("key", b"_cos\nsystem\n(S'echo hello world'\ntR.")
     with pytest.raises(UnSecureDataError):
         await cache.get_many("key")
-    assert not await cache.get_raw("key")
 
 
 async def test_no_value(cache):
@@ -167,16 +181,14 @@ async def test_replace_values(cache):
 
     with pytest.raises(UnSecureDataError):
         await cache.get("replace")
-    assert not await cache.get_raw("replace")
 
 
 async def test_pickle_error_value(cache):
     await cache.set_raw(
         "key",
-        cache.get_sign("key", b"no_pickle_data", b"md5") + b"_" + b"no_pickle_data",
+        cache._get_sign("key", b"no_pickle_data", b"md5") + b"_" + b"no_pickle_data",
     )
     assert await cache.get("key") is None
-    assert not await cache.get_raw("key")
 
 
 async def test_set_no_ser(cache):
@@ -184,15 +196,6 @@ async def test_set_no_ser(cache):
     await cache.set("key_e", empty)
 
 
-async def test_no_import_dc(cache):
-    @dataclasses.dataclass
-    class TestNoImport:
-        test: str
-
-    with pytest.raises(AttributeError):
-        await cache.set("key_i", TestNoImport(test="test"))
-
-
 Schema = None
 
 
@@ -222,3 +225,12 @@ async def test_data_change(cache):
 async def test_get_set_raw(cache):
     await cache.set_raw("key", b"test")
     assert await cache.get_raw("key") == b"test"
+
+
+@given(key=st.text(), value=st.characters())
+@settings(max_examples=500)
+@example(key="_key:_!@#$%^&*()", value='_value:_!@_#$%^&:*(?)".,4й')
+async def test_no_hash(key, value):
+    cache = Cache()
+    await cache.set(key, value)
+    assert await cache.get(key) == value
