@@ -1,7 +1,7 @@
 import pytest
 
+from cashews._settings import BackendNotAvailable, settings_url_parse
 from cashews.backends.memory import Memory
-from cashews.wrapper import BackendNotAvailable, settings_url_parse
 
 
 @pytest.mark.parametrize(
@@ -24,7 +24,7 @@ def test_url(url, params):
     (
         (
             "redis://localhost:9000/0",
-            "Redis backend requires `aioredis` to be installed.",
+            "Redis backend requires `redis` (or `aioredis`) to be installed.",
         ),
         ("disk://", "Disk backend requires `diskcache` to be installed."),
     ),
@@ -57,7 +57,7 @@ def test_url_but_backend_dependency_is_not_installed(url, error):
             },
         ),
         (
-            "redis://localhost/0/?hash_key=secret&password=test&safe=1&minsize=3&create_connection_timeout=0.1",
+            "redis://localhost/0/?hash_key=secret&password=test&safe=1&minsize=3&create_connection_timeout=0.1&pickle_type=dill",  # noqa: E501
             {
                 "backend": None,
                 "address": "redis://localhost/0/",
@@ -66,6 +66,7 @@ def test_url_but_backend_dependency_is_not_installed(url, error):
                 "safe": True,
                 "minsize": 3,
                 "create_connection_timeout": 0.1,
+                "pickle_type": "dill",
             },
         ),
         (
