@@ -3,10 +3,10 @@ from unittest.mock import Mock, PropertyMock
 
 import pytest
 
+from cashews._cache_condition import NOT_NONE
 from cashews.backends.memory import Memory
 from cashews.disable_control import ControlMixin
 from cashews.formatter import get_templates_for_func
-from cashews.helpers import add_prefix, all_keys_lower
 from cashews.wrapper import Cache, _create_auto_init
 
 pytestmark = pytest.mark.asyncio
@@ -43,10 +43,23 @@ async def test_prefix(cache):
     await cache.set("-:key", "-value")
 
     assert await cache.get("key") == "value"
-    assert await cache.get("-:key") == None
+    assert await cache.get("-:key") is None
     assert await cache.get("-:key", default="def") == "def"
 
 
+async def test_prefix_many(cache):
+    await cache.init("mem://")
+    await cache.init("mem://", prefix="-")
+
+    await cache.set("key", "value")
+    await cache.set("-:key", "-value")
+
+    assert await cache.get("key") == "value"
+    assert await cache.get("-:key") == "-value"
+
+    assert await cache.get_many("key", "-:key") == ("value", "-value")
+
+
 async def test_disable_cmd(cache):
     await cache.init("mem://localhost")
     cache.disable("incr")
@@ -80,6 +93,7 @@ async def test_disable_decorators(cache: Cache, target):
     data = (i for i in range(10))
 
     @cache(ttl=1)
+    @cache.soft(ttl=1)
     @cache.failover(ttl=1)
     @cache.hit(ttl=1, cache_hits=1)
     @cache.circuit_breaker(ttl=1, errors_rate=1, period=1)
@@ -164,7 +178,7 @@ async def test_disable_decorators_set(cache: Cache):
 
 async def test_init(cache):
     await cache.init("mem://localhost")
-    assert cache.is_enable()
+    assert cache.is_init
 
 
 async def test_auto_init(cache):
@@ -186,51 +200,6 @@ async def test_auto_init(cache):
     target.init.assert_called_once()
 
 
-async def test_all_keys_lower(cache: Cache, target):
-    cache._backends[""] = cache._backends[""][0], (all_keys_lower(),)
-    await cache.get(key="KEY")
-    target.get.assert_called_once_with(key="key", default=None)
-
-    await cache.set(key="KEY", value="value")
-    target.set.assert_called_once_with(
-        key="key",
-        value="value",
-        exist=None,
-        expire=None,
-    )
-    await cache.ping()
-    target.ping.assert_called_once_with(message=b"PING")
-
-
-async def test_add_prefix(cache: Cache, target):
-    cache._backends[""] = cache._backends[""][0], (add_prefix("prefix!"),)
-
-    await cache.get(key="key")
-    target.get.assert_called_once_with(key="prefix!key", default=None)
-
-    await cache.set(key="key", value="value")
-    target.set.assert_called_once_with(
-        key="prefix!key",
-        value="value",
-        exist=None,
-        expire=None,
-    )
-    await cache.ping()
-    target.ping.assert_called_once_with(message=b"PING")
-
-
-async def test_add_prefix_get_many(cache: Cache, target):
-    cache._backends[""] = cache._backends[""][0], (add_prefix("prefix!"),)
-    await cache.get_many("key")
-    target.get_many.assert_called_once_with("prefix!key")
-
-
-async def test_add_prefix_delete_match(cache: Cache, target):
-    cache._backends[""] = cache._backends[""][0], (add_prefix("prefix!"),)
-    await cache.delete_match("key")
-    target.delete_match.assert_called_once_with(pattern="prefix!key")
-
-
 async def test_smoke_cmds(cache: Cache, target):
     await cache.set(key="key", value={"any": True}, expire=60, exist=None)
     target.set.assert_called_once_with(key="key", value={"any": True}, expire=60, exist=None)
@@ -238,8 +207,11 @@ async def test_smoke_cmds(cache: Cache, target):
     await cache.get("key")  # -> Any
     target.get.assert_called_once_with(key="key", default=None)
 
+    await cache.set_many({"key1": "value1", "key2": "value2"}, expire=60)
+    target.set_many.assert_called_once_with({"key1": "value1", "key2": "value2"}, expire=60)
+
     await cache.get_many("key1", "key2")
-    target.get_many.assert_called_once_with("key1", "key2")
+    target.get_many.assert_called_once_with("key1", "key2", default=None)
 
     await cache.incr("key_incr")  # -> int
     target.incr.assert_called_once_with(key="key_incr")
@@ -278,9 +250,11 @@ async def test_smoke_cmds(cache: Cache, target):
     target.incr_bits.assert_called_once_with("key", 1, 2, 3, size=2, by=1)
 
     [key async for key in cache.keys_match("key:*")]
-
     target.keys_match.assert_called_once_with("key:*")
 
+    [key_value async for key_value in cache.get_match("key:*")]
+    target.get_match.assert_called_once_with("key:*", batch_size=100, default=None)
+
 
 async def test_disable_cache_on_fail_return(cache: Cache):
     assert await cache.get("key") is None
@@ -364,3 +338,49 @@ async def test_cache_lock():
     await asyncio.gather(my_func(), my_func(), my_func())
 
     m.assert_called_once_with(1)
+
+
+_cache = Cache()
+_cache.setup("mem://")
+
+
+@pytest.mark.parametrize("decorator", (_cache, _cache.dynamic, _cache.early, _cache.soft))
+async def test_time_condition(decorator):
+    m = Mock()
+
+    @decorator(key="test", ttl=10, time_condition=0.1)
+    async def my_func(sleep=0.01):
+        await asyncio.sleep(sleep)
+        m()
+        return sleep
+
+    await my_func()
+    await asyncio.gather(my_func(), my_func(), my_func())
+    assert m.call_count == 4
+    m.reset_mock()
+
+    await asyncio.gather(my_func(0.15), my_func(), my_func(0.1))
+    assert m.call_count == 3
+    await my_func(0.15)
+    await my_func()
+    assert m.call_count == 3
+
+    m.reset_mock()
+    await _cache.clear()
+    await my_func()
+    assert m.call_count == 1
+
+
+async def test_cache_simple_not_none(cache: Cache):
+    mock = Mock()
+
+    @cache(ttl=0.1, key="key", condition=NOT_NONE)
+    async def func():
+        mock()
+        return None
+
+    assert await func() is None
+    assert mock.call_count == 1
+
+    assert await func() is None
+    assert mock.call_count == 2
